==> braze_plugin-0.3.0.tar.gz: lib/braze_plugin.dart <==
import 'dart:async';
import 'dart:convert' as json;
import 'package:flutter/services.dart';

class BrazePlugin {
  static const MethodChannel _channel = const MethodChannel('braze_plugin');
  Function(BrazeInAppMessage) _brazeInAppMessageHandler;

  BrazePlugin() {
    _channel.setMethodCallHandler(_handleBrazeData);
  }

  /// Sets a callback to receive in-app message data from Braze
  void setBrazeInAppMessageCallback(Function(BrazeInAppMessage) callback) {
    _brazeInAppMessageHandler = callback;
  }

  /// Changes the current Braze userId
  void changeUser(String userId) {
    final Map<String, dynamic> params = <String, dynamic>{"userId": userId};
    _channel.invokeMethod('changeUser', params);
  }

  /// Logs a click for the provided in-app message data
  void logInAppMessageClicked(BrazeInAppMessage inAppMessage) {
    final Map<String, dynamic> params = <String, dynamic>{
      "inAppMessageString": inAppMessage.inAppMessageJsonString
    };
    _channel.invokeMethod('logInAppMessageClicked', params);
  }

  /// Logs an impression for the provided in-app message data
  void logInAppMessageImpression(BrazeInAppMessage inAppMessage) {
    final Map<String, dynamic> params = <String, dynamic>{
      "inAppMessageString": inAppMessage.inAppMessageJsonString
    };
    _channel.invokeMethod('logInAppMessageImpression', params);
  }

  /// Logs a button click for the provided in-app message button data
  void logInAppMessageButtonClicked(BrazeInAppMessage inAppMessage,
      int buttonId) {
    final Map<String, dynamic> params = <String, dynamic>{
      "inAppMessageString": inAppMessage.inAppMessageJsonString,
      "buttonId": buttonId
    };
    _channel.invokeMethod('logInAppMessageButtonClicked', params);
  }
  
  /// Gets the install tracking id
  Future<String> getInstallTrackingId() {
    return _channel.invokeMethod('getInstallTrackingId')
        .then<String>((dynamic result) => result);
  }

  /// Add alias for current user
  void addAlias(String aliasName, String aliasLabel) {
    final Map<String, dynamic> params = <String, dynamic>{
      'aliasName': aliasName,
      'aliasLabel': aliasLabel
    };
    _channel.invokeMethod('addAlias', params);
  }

  /// Logs a custom event to Braze
  void logCustomEvent(String eventName) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName": eventName
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  /// Logs a custom event to Braze
  void logCustomEventWithProperties(
      String eventName, Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName": eventName,
      "properties": properties
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  /// Logs a purchase event to Braze
  void logPurchase(
      String productId, String currencyCode, double price, int quantity) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId": productId,
      "currencyCode": currencyCode,
      "price": price,
      "quantity": quantity
    };
    _channel.invokeMethod('logPurchase', params);
  }

  /// Logs a purchase event to Braze
  void logPurchaseWithProperties(String productId, String currencyCode,
      double price, int quantity, Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId": productId,
      "currencyCode": currencyCode,
      "price": price,
      "quantity": quantity,
      "properties": properties
    };
    _channel.invokeMethod('logPurchase', params);
  }

  /// Adds an element to a custom attribute array
  void addToCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('addToCustomAttributeArray', params);
  }

  /// Removes an element from a custom attribute array
  void removeFromCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('removeFromCustomAttributeArray', params);
  }

  /// Sets a string typed custom attribute
  void setStringCustomUserAttribute(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setStringCustomUserAttribute', params);
  }

  /// Sets a double typed custom attribute
  void setDoubleCustomUserAttribute(String key, double value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setDoubleCustomUserAttribute', params);
  }

  /// Sets a boolean typed custom attribute
  void setBoolCustomUserAttribute(String key, bool value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setBoolCustomUserAttribute', params);
  }

  /// Sets a integer typed custom attribute
  void setIntCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setIntCustomUserAttribute', params);
  }

  /// Increments an integer typed custom attribute
  void incrementCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('incrementCustomUserAttribute', params);
  }

  /// Sets a location custom attribute
  void setLocationCustomAttribute(String key, double lat, double long) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'lat': lat,
      'long': long
    };
    _channel.invokeMethod('setLocationCustomAttribute', params);
  }

  /// Sets a date custom attribute
  void setDateCustomUserAttribute(String key, DateTime value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value.millisecondsSinceEpoch ~/ 1000
    };
    _channel.invokeMethod('setDateCustomUserAttribute', params);
  }

  /// Unsets a custom attribute
  void unsetCustomUserAttribute(String key) {
    _callStringMethod('unsetCustomUserAttribute', 'key', key);
  }

  /// Sets the first name default user attribute
  void setFirstName(String firstName) {
    _callStringMethod('setFirstName', 'firstName', firstName);
  }

  /// Sets the last name default user attribute
  void setLastName(String lastName) {
    _callStringMethod('setLastName', 'lastName', lastName);
  }

  /// Sets the email default user attribute
  void setEmail(String email) {
    _callStringMethod('setEmail', 'email', email);
  }

  /// Sets the dob default user attribute
  void setDateOfBirth(int year, int month, int day) {
    final Map<String, dynamic> params = <String, dynamic>{
      'year': year,
      'month': month,
      'day': day
    };
    _channel.invokeMethod('setDateOfBirth', params);
  }

  /// Sets the gender default user attribute
  void setGender(String gender) {
    _callStringMethod('setGender', 'gender', gender);
  }

  /// Sets the language default user attribute
  void setLanguage(String language) {
    _callStringMethod('setLanguage', 'language', language);
  }

  /// Sets the country default user attribute
  void setCountry(String country) {
    _callStringMethod('setCountry', 'country', country);
  }

  /// Sets the home city default user attribute
  void setHomeCity(String homeCity) {
    _callStringMethod('setHomeCity', 'homeCity', homeCity);
  }

  /// Sets the phone number default user attribute
  void setPhoneNumber(String phoneNumber) {
    _callStringMethod('setPhoneNumber', 'phoneNumber', phoneNumber);
  }

  /// Sets the avatar image url default user attribute
  void setAvatarImageUrl(String avatarImageUrl) {
    _callStringMethod('setAvatarImageUrl', 'avatarImageUrl', avatarImageUrl);
  }

  /// Sets attribution data
  void setAttributionData(
      String network, String campaign, String adGroup, String creative) {
    final Map<String, dynamic> params = <String, dynamic>{
      'network': network,
      'campaign': campaign,
      'adGroup': adGroup,
      'creative': creative
    };
    _channel.invokeMethod('setAttributionData', params);
  }

  /// Registers a push token for the current device with Braze
  void registerPushToken(String pushToken) {
    _callStringMethod('registerPushToken', 'pushToken', pushToken);
  }

  /// Requests an immediate data flush
  void requestImmediateDataFlush() {
    _channel.invokeMethod('requestImmediateDataFlush');
  }

  /// Wipes Data on board the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void wipeData() {
    _channel.invokeMethod('wipeData');
  }

  /// Requests location initialization
  void requestLocationInitialization() {
    _channel.invokeMethod('requestLocationInitialization');
  }

  /// Enables the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void enableSDK() {
    _channel.invokeMethod('enableSDK');
  }

  /// Disables the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void disableSDK() {
    _channel.invokeMethod('disableSDK');
  }

  /// Sets push subscription state for the current user
  void setPushNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type': type.toString()
    };
    _channel.invokeMethod('setPushNotificationSubscriptionType', params);
  }

  /// Sets email subscription state for the current user
  void setEmailNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type': type.toString()
    };
    _channel.invokeMethod('setEmailNotificationSubscriptionType', params);
  }

  void _callStringMethod(String methodName, String paramName, String value) {
    final Map<String, dynamic> params = <String, dynamic>{paramName: value};
    _channel.invokeMethod(methodName, params);
  }

  Future<dynamic> _handleBrazeData(MethodCall call) async {
    switch(call.method) {
      case "handleBrazeInAppMessage":
        final Map<dynamic, dynamic> argumentsMap = call.arguments;
        String inAppMessageString = argumentsMap['inAppMessage'];
        if (_brazeInAppMessageHandler is Function(BrazeInAppMessage)) {
          _brazeInAppMessageHandler(BrazeInAppMessage(inAppMessageString));
        } else {
          print("Braze in-app message callback not present. Doing nothing.");
        }
        return new Future.value("");
      default:
        print("Unknown method " + call.method + " called. Doing nothing.");
        return new Future.value("");

    }
  }
}

enum SubscriptionType { subscribed, unsubscribed, opted_in }

/// Braze in-app message dismiss types
///
/// Messages with a dismiss type of "swipe" should appear until manually
/// dismissed by the  user. Messages with a dismiss type of "auto_dismiss"
/// should dismiss automatically after the amount of time specified in the
/// "duration" field of the message's BrazeInAppMessage object elapses.
enum DismissType { swipe, auto_dismiss }

/// Braze in-app message click actions
///
/// Messages with a "uri" click action should navigate to the specified uri
/// while respecting the "useWebView" field of the message's BrazeInAppMessage
/// object if appropriate.
enum ClickAction { news_feed, uri, none }

/// Braze in-app message types
enum MessageType { slideup, modal, full, html_full }

class BrazeInAppMessage {
  /// In-app message text
  String message = "";

  /// In-app message header
  String header = "";

  /// Url of the image to display alongside the message
  String imageUrl = "";

  /// Uri to navigate to on-click
  String uri = "";

  /// Whether to use a webview to display the uri when clicked
  bool useWebView = false;

  /// In-app message display duration in milliseconds
  int duration = 5;

  /// In-app message body click action
  ClickAction clickAction = ClickAction.none;

  /// In-app message dismiss type
  DismissType dismissType = DismissType.auto_dismiss;

  /// In-app message type
  MessageType messageType = MessageType.slideup;

  /// Buttons to display alongside this in-app message
  List<BrazeButton> buttons = List<BrazeButton>();

  /// Key-value pair extras
  Map<String, String> extras = Map();

  /// In-app message json
  String inAppMessageJsonString = "";

  BrazeInAppMessage(String _data) {
    inAppMessageJsonString = _data;
    var inAppMessageJson = json.jsonDecode(_data);

    var messageJson = inAppMessageJson["message"];
    if (messageJson is String) {
      message = messageJson;
    }
    var headerJson = inAppMessageJson["header"];
    if (headerJson is String) {
      header = headerJson;
    }
    var uriJson = inAppMessageJson["uri"];
    if (uriJson is String) {
      uri = uriJson;
    }
    var imageUrlJson = inAppMessageJson["image_url"];
    if (imageUrlJson is String) {
      imageUrl = imageUrlJson;
    }
    var useWebViewJson = inAppMessageJson["use_webview"];
    if (useWebViewJson is bool) {
      useWebView = useWebViewJson;
    }
    var durationJson = inAppMessageJson["duration"];
    if (durationJson is int) {
      duration = durationJson;
    }
    var clickActionJson = inAppMessageJson["click_action"];
    if (clickActionJson is String) {
      for (ClickAction action in ClickAction.values) {
        if (action.toString().toLowerCase().endsWith(clickActionJson
            .toLowerCase())) {
          clickAction = action;
        }
      }
    }
    var dismissTypeJson = inAppMessageJson["message_close"];
    if (dismissTypeJson is String) {
      for (DismissType type in DismissType.values) {
        if (type.toString().toLowerCase().endsWith(dismissTypeJson
            .toLowerCase())) {
          dismissType = type;
        }
      }
    }
    var messageTypeJson = inAppMessageJson["type"];
    if (messageTypeJson is String) {
      for (MessageType type in MessageType.values) {
        if (type.toString().toLowerCase().endsWith(messageTypeJson
            .toLowerCase())) {
          messageType = type;
        }
      }
    }
    var extrasJson = inAppMessageJson["extras"];
    if (extrasJson is Map<String, dynamic>) {
      extrasJson.forEach((key, value) {
        if (extrasJson[key] is String) {
          extras[key] = value;
        }
      });
    }
    var buttonsJson = inAppMessageJson["btns"];
    if (buttonsJson is List<dynamic>) {
      for (var buttonJson in buttonsJson) {
        buttons.add(BrazeButton(buttonJson));
      }
    }
  }

  @override
  String toString() {
    return inAppMessageJsonString;
  }
}

class BrazeButton {
  /// Button text
  String text = "";

  /// Uri to navigate to on-click
  String uri = "";

  /// Whether to use a webview to display the uri when clicked
  bool useWebView = false;

  /// Button click action
  ClickAction clickAction = ClickAction.none;

  /// Id for analytics
  int id = 0;

  BrazeButton(dynamic buttonJson) {
    var textJson = buttonJson["text"];
    if (textJson is String) {
      text = textJson;
    }
    var uriJson = buttonJson["uri"];
    if (uriJson is String) {
      uri = uriJson;
    }
    var useWebViewJson = buttonJson["use_webview"];
    if (useWebViewJson is bool) {
      useWebView = useWebViewJson;
    }
    var clickActionJson = buttonJson["click_action"];
    if (clickActionJson is String) {
      for (ClickAction action in ClickAction.values) {
        if (action.toString().toLowerCase().endsWith(clickActionJson
            .toLowerCase())) {
          clickAction = action;
        }
      }
    }
    var idJson = buttonJson["id"];
    if (idJson is int) {
      id = idJson;
    }
  }

  @override
  String toString() {
    return "BrazeButton text:" + text + " uri:" + uri + " clickAction:"
        + clickAction.toString() + " useWebView: " + useWebView.toString();
  }
}

==> braze_plugin-0.6.0.tar.gz: lib/braze_plugin.dart <==
import 'dart:async';
import 'dart:convert' as json;
import 'package:flutter/services.dart';

class BrazePlugin {
  static const MethodChannel _channel = const MethodChannel('braze_plugin');
  Function(BrazeInAppMessage) _brazeInAppMessageHandler;

  BrazePlugin() {
    _channel.setMethodCallHandler(_handleBrazeData);
  }

  /// Sets a callback to receive in-app message data from Braze
  void setBrazeInAppMessageCallback(Function(BrazeInAppMessage) callback) {
    _brazeInAppMessageHandler = callback;
  }

  /// Changes the current Braze userId
  void changeUser(String userId) {
    final Map<String, dynamic> params = <String, dynamic>{"userId": userId};
    _channel.invokeMethod('changeUser', params);
  }

  /// Logs a click for the provided in-app message data
  void logInAppMessageClicked(BrazeInAppMessage inAppMessage) {
    final Map<String, dynamic> params = <String, dynamic>{
      "inAppMessageString": inAppMessage.inAppMessageJsonString
    };
    _channel.invokeMethod('logInAppMessageClicked', params);
  }

  /// Logs an impression for the provided in-app message data
  void logInAppMessageImpression(BrazeInAppMessage inAppMessage) {
    final Map<String, dynamic> params = <String, dynamic>{
      "inAppMessageString": inAppMessage.inAppMessageJsonString
    };
    _channel.invokeMethod('logInAppMessageImpression', params);
  }

  /// Logs a button click for the provided in-app message button data
  void logInAppMessageButtonClicked(BrazeInAppMessage inAppMessage,
      int buttonId) {
    final Map<String, dynamic> params = <String, dynamic>{
      "inAppMessageString": inAppMessage.inAppMessageJsonString,
      "buttonId": buttonId
    };
    _channel.invokeMethod('logInAppMessageButtonClicked', params);
  }
  
  /// Gets the install tracking id
  Future<String> getInstallTrackingId() {
    return _channel.invokeMethod('getInstallTrackingId')
        .then<String>((dynamic result) => result);
  }

  /// Add alias for current user
  void addAlias(String aliasName, String aliasLabel) {
    final Map<String, dynamic> params = <String, dynamic>{
      'aliasName': aliasName,
      'aliasLabel': aliasLabel
    };
    _channel.invokeMethod('addAlias', params);
  }

  /// Logs a custom event to Braze
  void logCustomEvent(String eventName) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName": eventName
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  /// Logs a custom event to Braze
  void logCustomEventWithProperties(
      String eventName, Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName": eventName,
      "properties": properties
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  /// Logs a purchase event to Braze
  void logPurchase(
      String productId, String currencyCode, double price, int quantity) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId": productId,
      "currencyCode": currencyCode,
      "price": price,
      "quantity": quantity
    };
    _channel.invokeMethod('logPurchase', params);
  }

  /// Logs a purchase event to Braze
  void logPurchaseWithProperties(String productId, String currencyCode,
      double price, int quantity, Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId": productId,
      "currencyCode": currencyCode,
      "price": price,
      "quantity": quantity,
      "properties": properties
    };
    _channel.invokeMethod('logPurchase', params);
  }

  /// Adds an element to a custom attribute array
  void addToCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('addToCustomAttributeArray', params);
  }

  /// Removes an element from a custom attribute array
  void removeFromCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('removeFromCustomAttributeArray', params);
  }

  /// Sets a string typed custom attribute
  void setStringCustomUserAttribute(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setStringCustomUserAttribute', params);
  }

  /// Sets a double typed custom attribute
  void setDoubleCustomUserAttribute(String key, double value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setDoubleCustomUserAttribute', params);
  }

  /// Sets a boolean typed custom attribute
  void setBoolCustomUserAttribute(String key, bool value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setBoolCustomUserAttribute', params);
  }

  /// Sets a integer typed custom attribute
  void setIntCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setIntCustomUserAttribute', params);
  }

  /// Increments an integer typed custom attribute
  void incrementCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('incrementCustomUserAttribute', params);
  }

  /// Sets a location custom attribute
  void setLocationCustomAttribute(String key, double lat, double long) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'lat': lat,
      'long': long
    };
    _channel.invokeMethod('setLocationCustomAttribute', params);
  }

  /// Sets a date custom attribute
  void setDateCustomUserAttribute(String key, DateTime value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value.millisecondsSinceEpoch ~/ 1000
    };
    _channel.invokeMethod('setDateCustomUserAttribute', params);
  }

  /// Unsets a custom attribute
  void unsetCustomUserAttribute(String key) {
    _callStringMethod('unsetCustomUserAttribute', 'key', key);
  }

  /// Sets the first name default user attribute
  void setFirstName(String firstName) {
    _callStringMethod('setFirstName', 'firstName', firstName);
  }

  /// Sets the last name default user attribute
  void setLastName(String lastName) {
    _callStringMethod('setLastName', 'lastName', lastName);
  }

  /// Sets the email default user attribute
  void setEmail(String email) {
    _callStringMethod('setEmail', 'email', email);
  }

  /// Sets the dob default user attribute
  void setDateOfBirth(int year, int month, int day) {
    final Map<String, dynamic> params = <String, dynamic>{
      'year': year,
      'month': month,
      'day': day
    };
    _channel.invokeMethod('setDateOfBirth', params);
  }

  /// Sets the gender default user attribute
  void setGender(String gender) {
    _callStringMethod('setGender', 'gender', gender);
  }

  /// Sets the language default user attribute
  void setLanguage(String language) {
    _callStringMethod('setLanguage', 'language', language);
  }

  /// Sets the country default user attribute
  void setCountry(String country) {
    _callStringMethod('setCountry', 'country', country);
  }

  /// Sets the home city default user attribute
  void setHomeCity(String homeCity) {
    _callStringMethod('setHomeCity', 'homeCity', homeCity);
  }

  /// Sets the phone number default user attribute
  void setPhoneNumber(String phoneNumber) {
    _callStringMethod('setPhoneNumber', 'phoneNumber', phoneNumber);
  }

  /// Sets the avatar image url default user attribute
  void setAvatarImageUrl(String avatarImageUrl) {
    _callStringMethod('setAvatarImageUrl', 'avatarImageUrl', avatarImageUrl);
  }

  /// Sets attribution data
  void setAttributionData(
      String network, String campaign, String adGroup, String creative) {
    final Map<String, dynamic> params = <String, dynamic>{
      'network': network,
      'campaign': campaign,
      'adGroup': adGroup,
      'creative': creative
    };
    _channel.invokeMethod('setAttributionData', params);
  }

  /// Registers a push token for the current Android device with Braze
  /// No-op on iOS.
  void registerAndroidPushToken(String pushToken) {
    _callStringMethod('registerAndroidPushToken', 'pushToken', pushToken);
  }

  /// Requests an immediate data flush
  void requestImmediateDataFlush() {
    _channel.invokeMethod('requestImmediateDataFlush');
  }

  /// Wipes Data on board the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void wipeData() {
    _channel.invokeMethod('wipeData');
  }

  /// Requests location initialization
  void requestLocationInitialization() {
    _channel.invokeMethod('requestLocationInitialization');
  }

  /// Enables the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void enableSDK() {
    _channel.invokeMethod('enableSDK');
  }

  /// Disables the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void disableSDK() {
    _channel.invokeMethod('disableSDK');
  }

  /// Sets push subscription state for the current user
  void setPushNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type': type.toString()
    };
    _channel.invokeMethod('setPushNotificationSubscriptionType', params);
  }

  /// Sets email subscription state for the current user
  void setEmailNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type': type.toString()
    };
    _channel.invokeMethod('setEmailNotificationSubscriptionType', params);
  }

  void _callStringMethod(String methodName, String paramName, String value) {
    final Map<String, dynamic> params = <String, dynamic>{paramName: value};
    _channel.invokeMethod(methodName, params);
  }

  Future<dynamic> _handleBrazeData(MethodCall call) async {
    switch(call.method) {
      case "handleBrazeInAppMessage":
        final Map<dynamic, dynamic> argumentsMap = call.arguments;
        String inAppMessageString = argumentsMap['inAppMessage'];
        if (_brazeInAppMessageHandler is Function(BrazeInAppMessage)) {
          _brazeInAppMessageHandler(BrazeInAppMessage(inAppMessageString));
        } else {
          print("Braze in-app message callback not present. Doing nothing.");
        }
        return new Future.value("");
      default:
        print("Unknown method " + call.method + " called. Doing nothing.");
        return new Future.value("");

    }
  }
}

enum SubscriptionType { subscribed, unsubscribed, opted_in }

/// Braze in-app message dismiss types
///
/// Messages with a dismiss type of "swipe" should appear until manually
/// dismissed by the  user. Messages with a dismiss type of "auto_dismiss"
/// should dismiss automatically after the amount of time specified in the
/// "duration" field of the message's BrazeInAppMessage object elapses.
enum DismissType { swipe, auto_dismiss }

/// Braze in-app message click actions
///
/// Messages with a "uri" click action should navigate to the specified uri
/// while respecting the "useWebView" field of the message's BrazeInAppMessage
/// object if appropriate.
enum ClickAction { news_feed, uri, none }

/// Braze in-app message types
enum MessageType { slideup, modal, full, html_full }

class BrazeInAppMessage {
  /// In-app message text
  String message = "";

  /// In-app message header
  String header = "";

  /// Url of the image to display alongside the message
  String imageUrl = "";

  /// Url of zipped assets to display with an HTML message
  String zippedAssetsUrl = "";

  /// Uri to navigate to on-click
  String uri = "";

  /// Whether to use a webview to display the uri when clicked
  bool useWebView = false;

  /// In-app message display duration in milliseconds
  int duration = 5;

  /// In-app message body click action
  ClickAction clickAction = ClickAction.none;

  /// In-app message dismiss type
  DismissType dismissType = DismissType.auto_dismiss;

  /// In-app message type
  MessageType messageType = MessageType.slideup;

  /// Buttons to display alongside this in-app message
  List<BrazeButton> buttons = List<BrazeButton>();

  /// Key-value pair extras
  Map<String, String> extras = Map();

  /// In-app message json
  String inAppMessageJsonString = "";

  BrazeInAppMessage(String _data) {
    inAppMessageJsonString = _data;
    var inAppMessageJson = json.jsonDecode(_data);

    var messageJson = inAppMessageJson["message"];
    if (messageJson is String) {
      message = messageJson;
    }
    var headerJson = inAppMessageJson["header"];
    if (headerJson is String) {
      header = headerJson;
    }
    var uriJson = inAppMessageJson["uri"];
    if (uriJson is String) {
      uri = uriJson;
    }
    var imageUrlJson = inAppMessageJson["image_url"];
    if (imageUrlJson is String) {
      imageUrl = imageUrlJson;
    }
    var zippedAssetsUrlJson = inAppMessageJson["zipped_assets_url"];
    if (zippedAssetsUrlJson is String) {
      zippedAssetsUrl = zippedAssetsUrlJson;
    }
    var useWebViewJson = inAppMessageJson["use_webview"];
    if (useWebViewJson is bool) {
      useWebView = useWebViewJson;
    }
    var durationJson = inAppMessageJson["duration"];
    if (durationJson is int) {
      duration = durationJson;
    }
    var clickActionJson = inAppMessageJson["click_action"];
    if (clickActionJson is String) {
      for (ClickAction action in ClickAction.values) {
        if (action.toString().toLowerCase().endsWith(clickActionJson
            .toLowerCase())) {
          clickAction = action;
        }
      }
    }
    var dismissTypeJson = inAppMessageJson["message_close"];
    if (dismissTypeJson is String) {
      for (DismissType type in DismissType.values) {
        if (type.toString().toLowerCase().endsWith(dismissTypeJson
            .toLowerCase())) {
          dismissType = type;
        }
      }
    }
    var messageTypeJson = inAppMessageJson["type"];
    if (messageTypeJson is String) {
      for (MessageType type in MessageType.values) {
        if (type.toString().toLowerCase().endsWith(messageTypeJson
            .toLowerCase())) {
          messageType = type;
        }
      }
    }
    var extrasJson = inAppMessageJson["extras"];
    if (extrasJson is Map<String, dynamic>) {
      extrasJson.forEach((key, value) {
        if (extrasJson[key] is String) {
          extras[key] = value;
        }
      });
    }
    var buttonsJson = inAppMessageJson["btns"];
    if (buttonsJson is List<dynamic>) {
      for (var buttonJson in buttonsJson) {
        buttons.add(BrazeButton(buttonJson));
      }
    }
  }

  @override
  String toString() {
    return inAppMessageJsonString;
  }
}

class BrazeButton {
  /// Button text
  String text = "";

  /// Uri to navigate to on-click
  String uri = "";

  /// Whether to use a webview to display the uri when clicked
  bool useWebView = false;

  /// Button click action
  ClickAction clickAction = ClickAction.none;

  /// Id for analytics
  int id = 0;

  BrazeButton(dynamic buttonJson) {
    var textJson = buttonJson["text"];
    if (textJson is String) {
      text = textJson;
    }
    var uriJson = buttonJson["uri"];
    if (uriJson is String) {
      uri = uriJson;
    }
    var useWebViewJson = buttonJson["use_webview"];
    if (useWebViewJson is bool) {
      useWebView = useWebViewJson;
    }
    var clickActionJson = buttonJson["click_action"];
    if (clickActionJson is String) {
      for (ClickAction action in ClickAction.values) {
        if (action.toString().toLowerCase().endsWith(clickActionJson
            .toLowerCase())) {
          clickAction = action;
        }
      }
    }
    var idJson = buttonJson["id"];
    if (idJson is int) {
      id = idJson;
    }
  }

  @override
  String toString() {
    return "BrazeButton text:" + text + " uri:" + uri + " clickAction:"
        + clickAction.toString() + " useWebView: " + useWebView.toString();
  }
}
